/**================================================================================================
Hochschule Luzern - Technik & Architektur                                  Mikrocontroller - HS2012
Copyright 2012 Felix Rohrer                                                    http://hslu.ximit.ch
---------------------------------------------------------------------------------------------------
Name        : Uebung_3-3.c
Author      : Felix Rohrer <felix.rohrer@stud.hslu.ch>
Version     : 2012-09-23v1.0
Description : Wrter Zhlen
              a) Binrbaum erzeugen
			  b) Ausgabe in PostOrder & InOrder
=================================================================================================*/


#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#define MAXWORD 100
#define BUFSIZE 100
#define TXTFILE "text.txt"

struct tnode{
	char *word;
	int count;
	struct tnode *left;
	struct tnode *right;
};

/* siehe auch Buch K&R S. 77 */
char buf[BUFSIZE];	// Buffer fuer ungetch()
int bufp = 0;		// naechste freie Position fr buf


/* function def */
int getch(void);
void ungetch(int c);
int getword(char *word, int lim);
struct tnode* talloc(void);
struct tnode* addtree(struct tnode *, char *);
void treeprintinorder(struct tnode* p);
void treeprintpostorder(struct tnode* p);



/**
 * The main procedure to run the application.
 * @param argc Number of command line arguments
 * @param argv The forwarded command line arguments
 * @return Application return (error) code
 */
int main(int argc, char** argv)
{
	struct tnode *root;
	char word[MAXWORD];
	root = NULL;

	printf("File '%s' wird eingelesen...\n", TXTFILE);
	if (freopen(TXTFILE, "r", stdin) == NULL) {
		printf("kann File nicht ffnen\n");
	}

	while(getword(word, MAXWORD) != EOF) {
		if(isalpha(word[0])) {
			root = addtree(root, word);
		}
	}

	printf("\nInOrder...\n");
	treeprintinorder(root);

	printf("\nPostOrder...\n");
	treeprintpostorder(root);


	return(EXIT_SUCCESS);
}



int getch(void){ /* naechstes Zeichen holen */
return (bufp > 0 ? buf[--bufp] : getchar());
}


void ungetch(int c){
	if (bufp >= BUFSIZE)
		printf("ungetch: too many characters\n");
	else
		buf[bufp++] = c;
}

/* siehe auch Buch K&R S. 131 */
int getword(char *word, int lim){
	int c;
	char *w = word;
	while(isspace(c = getch()))
		;
	if (c != EOF)
		*w++ = c;
	if (!isalpha(c)) {
		*w = '\0';
		return(c);
	}

	for( ; --lim > 0; w++)
		if(!isalnum(*w = getch())){
			ungetch(*w);
			break;
		}
	*w ='\0';
	
	return word[0];
}


/** Gibt Zeiger auf Speicher zurck. */
struct tnode* talloc(void)
{
	return ((struct tnode*) malloc(sizeof(struct tnode)));
}


struct tnode* addtree(struct tnode* p, char* w)
{
	int cond;
	if (p == NULL) {
		p = talloc(); // einen neuen Knoten erzeugen
		p->word = strdup(w); // Wort kopieren (strdup)
		p->count = 1;
		p->left = NULL;
		p->right = NULL;
	}
	else if ((cond = strcmp(w, p->word)) == 0)
		p->count++; // Wort schon vorgekommen
	else if (cond < 0) // kleiner, links darunter
		p->left = addtree(p->left, w);
	else // grsser, rechts darunter
		p->right = addtree(p->right,w);
	
	return p; // gibt einen Zeiger zurck, // wo zuletzt eingefgt
}


void treeprintinorder(struct tnode* p)
{
	if (p != NULL){
		treeprintinorder(p->left);
		printf("%4d %s\n", p->count, p->word);
		treeprintinorder(p->right);
	}
}


void treeprintpostorder(struct tnode* p)
{
	if (p != NULL){
		printf("%4d %s\n", p->count, p->word);
		treeprintpostorder(p->left);
		treeprintpostorder(p->right);
	}
}


