/**================================================================================================
Hochschule Luzern - Technik & Architektur                                  Mikrocontroller - HS2012
Copyright 2012 Felix Rohrer                                                    http://hslu.ximit.ch
---------------------------------------------------------------------------------------------------
Name        : Uebung_3-4.c
Author      : Felix Rohrer <felix.rohrer@stud.hslu.ch>
Version     : 2012-09-30v1.0
Description : Structure Tree
              Erstellen Sie eine baumartige, dynamische Datenstruktur zum Verwalten von Sensoren
			  und dessen Messwerte. Dynamisch bedeutet hier, dass die Anzahl Sensoren und Anzahl
			  Kanle pro Sensor erst zur Laufzeit bekannt sind.
=================================================================================================*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>


typedef struct oakSensors_s* oakSensorsPtr_t;
typedef struct sensor_s* sensorPtr_t;
typedef struct channel_s* channelPtr_t;

typedef struct oakSensors_s {
	int nbrOfSensors;
	sensorPtr_t sensor;
} oakSensors_t;

typedef struct sensor_s {
	int serialNumber;
	int productID;
	char* deviceName;
	int nbrOfChannels;
	channelPtr_t channel;
} sensor_t;

typedef struct channel_s {
	float value;
	int unitId;
	char* unitString;
	char* channelName;
} channel_t;


void scanSensors(oakSensorsPtr_t oakSensors);
void printSensors(oakSensorsPtr_t oakSensors);
void setValues(oakSensorsPtr_t oakSensors, int nbrSensor, int nbrChannel, float value);
void setAllValues(oakSensorsPtr_t oakSensors);
float readValues(oakSensorsPtr_t oakSensors, int nbrSensor, int nbrChannel);
void printMeasurements(oakSensorsPtr_t oakSensors);

/**
 * The main procedure to run the application.
 * @param argc Number of command line arguments
 * @param argv The forwarded command line arguments
 * @return Application return (error) code
 */
int main(int argc, char** argv)
{
	// Scan Sensors
	oakSensors_t oakSensors;
	scanSensors(&oakSensors);

	// PrintOut Sensor Information
	printSensors(&oakSensors);

	printf("----------------------------------------------------------------\n");
	// setAllValues - fake Data
	setAllValues(&oakSensors);

	// printMeasurements / readValues
	printMeasurements(&oakSensors);

	

	fflush(stdin); //flushes the input stream and gets rid of '\n'
	printf("\n\nPress [Enter] to exit...");
	getchar();
	return(EXIT_SUCCESS);
}

void printMeasurements(oakSensorsPtr_t oakSensors)
{
	int nbrOfSensors, i;
	int nbrOfChannels, j;
	sensorPtr_t sensor;
	channelPtr_t channel;

	// print Sensor
	nbrOfSensors = oakSensors->nbrOfSensors;
	for (i = 0; i < nbrOfSensors; i++) {
		sensor = oakSensors->sensor;
		nbrOfChannels = sensor[i].nbrOfChannels;
		printf("- %s:\n", sensor[i].deviceName);
		// print Channel, Value
		channel = sensor[i].channel;
		for (j = 0; j < nbrOfChannels; j++) {
			printf("\t- %s: %f %s\n", channel[j].channelName, channel[j].value, channel[j].unitString);
		}
	printf("\n");
	}
}


float readValues(oakSensorsPtr_t oakSensors, int nbrSensor, int nbrChannel)
{
	return oakSensors->sensor[nbrSensor].channel[nbrChannel].value;
}



void setAllValues(oakSensorsPtr_t oakSensors)
{
	// Atmospheric preasure
	setValues(oakSensors, 0, 0, 101539.0f);
	setValues(oakSensors, 0, 1, 297.700012f);
	// Humidity
	setValues(oakSensors, 1, 0, 297.299988f);
	setValues(oakSensors, 1, 1, 58.700001f);
	// Acceleration
	setValues(oakSensors, 2, 0, 0.01f);
	setValues(oakSensors, 2, 1, -0.02f);
	setValues(oakSensors, 2, 2, 9.81f);
	// Current
	setValues(oakSensors, 3, 0, 0.0075f);
	// Luminosity
	setValues(oakSensors, 4, 0, 12746.0f);
}


void setValues(oakSensorsPtr_t oakSensors, int nbrSensor, int nbrChannel, float value)
{
	oakSensors->sensor[nbrSensor].channel[nbrChannel].value = value;
}


void printSensors(oakSensorsPtr_t oakSensors)
{
	int nbrOfSensors, i;
	int nbrOfChannels, j;
	sensorPtr_t sensor;
	channelPtr_t channel;

	// print Sensor
	nbrOfSensors = oakSensors->nbrOfSensors;
	printf("%d Sensors\n", nbrOfSensors);
	for (i = 0; i < nbrOfSensors; i++) {
		sensor = oakSensors->sensor;
		nbrOfChannels = sensor[i].nbrOfChannels;
		printf("- %s (SN %d, PID %d) has %d channels\n", sensor[i].deviceName, sensor[i].serialNumber, sensor[i].productID, nbrOfChannels);
		// print Channel
		channel = sensor[i].channel;
		for (j = 0; j < nbrOfChannels; j++) {
			printf("\t- %s measures %s (%d)\n", channel[j].channelName, channel[j].unitString, channel[j].unitId);
		}
	printf("\n");
	}
}


void scanSensors(oakSensorsPtr_t oakSensors)
{
	const int nbrOfSensors = 5;
	const int nbrOfChannelsSensor0 = 2;
	const int nbrOfChannelsSensor1 = 2;
	const int nbrOfChannelsSensor2 = 3;
	const int nbrOfChannelsSensor3 = 1;
	const int nbrOfChannelsSensor4 = 1;

	// Allocate Memory
	sensorPtr_t sensor = (sensorPtr_t) malloc(sizeof(struct sensor_s) * nbrOfSensors);
	channelPtr_t channelSensor0 = (channelPtr_t) malloc(sizeof(struct channel_s) * nbrOfChannelsSensor0);
	channelPtr_t channelSensor1 = (channelPtr_t) malloc(sizeof(struct channel_s) * nbrOfChannelsSensor1);
	channelPtr_t channelSensor2 = (channelPtr_t) malloc(sizeof(struct channel_s) * nbrOfChannelsSensor2);
	channelPtr_t channelSensor3 = (channelPtr_t) malloc(sizeof(struct channel_s) * nbrOfChannelsSensor3);
	channelPtr_t channelSensor4 = (channelPtr_t) malloc(sizeof(struct channel_s) * nbrOfChannelsSensor4);

	// Add Sensor & Co to oakSensor Object
	oakSensors->nbrOfSensors = nbrOfSensors;
	oakSensors->sensor = sensor;

	// Sensor 0 -------------------------------------------------------------------------
	sensor[0].deviceName    = strdup("Atmospheric preasure");
	sensor[0].serialNumber  = 111111;
	sensor[0].productID     = 1;
	sensor[0].nbrOfChannels = nbrOfChannelsSensor0;
	sensor[0].channel       = channelSensor0;
	// Sensor 0 - Channel 0
	channelSensor0[0].channelName = strdup("Pressure");
	channelSensor0[0].unitString  = strdup("Pa");
	channelSensor0[0].unitId      = 23;
	// Sensor 0 - Channel 1
	channelSensor0[1].channelName = strdup("Temperature");
	channelSensor0[1].unitString  = strdup("K");
	channelSensor0[1].unitId      = 37;

	// Sensor 1 -------------------------------------------------------------------------
	sensor[1].deviceName    = strdup("Humidity");
	sensor[1].serialNumber  = 22222;
	sensor[1].productID     = 2;
	sensor[1].nbrOfChannels = nbrOfChannelsSensor1;
	sensor[1].channel       = channelSensor1;
	// Sensor 1 - Channel 0
	channelSensor1[0].channelName = strdup("Temperatur");
	channelSensor1[0].unitString  = strdup("K");
	channelSensor1[0].unitId      = 37;
	// Sensor 1 - Channel 1
	channelSensor1[1].channelName = strdup("Humidity");
	channelSensor1[1].unitString  = strdup("% relative");
	channelSensor1[1].unitId      = 49;

	// Sensor 2 -------------------------------------------------------------------------
	sensor[2].deviceName    = strdup("Acceleration");
	sensor[2].serialNumber  = 33333;
	sensor[2].productID     = 3;
	sensor[2].nbrOfChannels = nbrOfChannelsSensor2;
	sensor[2].channel       = channelSensor2;
	// Sensor 2 - Channel 0
	channelSensor2[0].channelName = strdup("x");
	channelSensor2[0].unitString  = strdup("m/s^2");
	channelSensor2[0].unitId      = 55;
	// Sensor 2 - Channel 1
	channelSensor2[1].channelName = strdup("y");
	channelSensor2[1].unitString  = strdup("m/s^2");
	channelSensor2[1].unitId      = 55;
	// Sensor 2 - Channel 2
	channelSensor2[2].channelName = strdup("z");
	channelSensor2[2].unitString  = strdup("m/s^2");
	channelSensor2[2].unitId      = 55;

	// Sensor 3 -------------------------------------------------------------------------
	sensor[3].deviceName    = strdup("Current");
	sensor[3].serialNumber  = 44444;
	sensor[3].productID     = 4;
	sensor[3].nbrOfChannels = nbrOfChannelsSensor3;
	sensor[3].channel       = channelSensor3;
	// Sensor 3 - Channel 0
	channelSensor3[0].channelName = strdup("Current (4..20 mA)");
	channelSensor3[0].unitString  = strdup("A");
	channelSensor3[0].unitId      = 13;

	// Sensor 4 -------------------------------------------------------------------------
	sensor[4].deviceName    = strdup("Luminosity");
	sensor[4].serialNumber  = 55555;
	sensor[4].productID     = 5;
	sensor[4].nbrOfChannels = nbrOfChannelsSensor4;
	sensor[4].channel       = channelSensor4;
	// Sensor 4 - Channel 0
	channelSensor4[0].channelName = strdup("Illuminance");
	channelSensor4[0].unitString  = strdup("Lux");
	channelSensor4[0].unitId      = 87;
}